type MethodArray = [
    'webkitRequestFullscreen',
    'requestFullscreen',
    'msRequestFullscreen',
    'mozRequestFullScreen'
];

type MethodName =
    'webkitRequestFullscreen' |
    'requestFullscreen' |
    'msRequestFullscreen' |
    'mozRequestFullScreen';

const _docEl = document.documentElement;

/**
 * 获取支持的"requestFullscreen"方法名
 */
export function _getMethodName(): MethodName {
    const _names: MethodArray = [
        'webkitRequestFullscreen',
        'requestFullscreen',
        'msRequestFullscreen',
        'mozRequestFullScreen'
    ];


    let methodName: MethodName = '' as MethodName;
    for (const name of _names) {
        if (undefined !== _docEl[name]) {
            methodName = name;
            break;
        }
    }
    return methodName;
}

/**
 * 启用全屏
 * @param  {HTMLElement} 元素
 * @param  {FullscreenOptions} 选项
 */
export function beFull(el: HTMLElement = _docEl, options?: FullscreenOptions) {
    const methodName = _getMethodName();
    return el[methodName](options);
}

/**
 * 退出全屏
 */
export function exitFull() {
    document.exitFullscreen();
}

/**
 * 切换全屏/关闭
 * @param  {HTMLElement} 目标元素
 */
export function toggleFull(el: HTMLElement = _docEl, options?: FullscreenOptions) {
    if (document.fullscreenElement) {
        document.exitFullscreen();
    } else {
        const methodName = _getMethodName();
        console.log(methodName);
        return el[methodName](options);
    }
}

/**
 * 当全屏/退出时触发
 * @param  {HTMLElement} 元素
 * @param  {(isFull: boolean) => void} 返回"是否全屏"
 */
export function watchFull(el: HTMLElement, callback: (isFull: boolean) => void) {
    const handler = (event: Event) => {
        callback(document.fullscreenElement === event.target);
    }
    el.addEventListener('onfullscreenchange', handler);
    return () => {
        el.removeEventListener('onfullscreenchange', handler);
    }
}